package org.netkernel.request.impl;

import java.util.*;
import org.netkernel.request.IRequestResponseFields;

public class RequestResponseFieldsWithStickyImpl implements IRequestResponseFields
{
	private Object[] mData;
	private int mIndex;
	
	public static final RequestResponseFieldsWithStickyImpl EMPTY = new RequestResponseFieldsWithStickyImpl(0);
	private static final int LEN=3;
	
	
	public static RequestResponseFieldsWithStickyImpl getSticky(IRequestResponseFields aOther)
	{
		RequestResponseFieldsWithStickyImpl result = new RequestResponseFieldsWithStickyImpl(aOther.size());
		if (aOther.size()==0)
		{	result=RequestResponseFieldsWithStickyImpl.EMPTY;
		}
		if (aOther instanceof RequestResponseFieldsWithStickyImpl)
		{	RequestResponseFieldsWithStickyImpl other = (RequestResponseFieldsWithStickyImpl)aOther;
			for (int i=0; i<other.mIndex; i+=LEN)
			{	boolean isSticky=(Boolean)other.mData[i+2];
				if (isSticky)
				{	String key=(String)other.mData[i];
					result.put(key, other.mData[i+1], true);
				}
			}
			if (result.size()==0)
			{	result=RequestResponseFieldsWithStickyImpl.EMPTY;
			}
			else if (result.size()==other.size())
			{	result=other;
			}
		}
		else
		{	int size=aOther.size();
			for (int i=0; i<size; i++)
			{	boolean isSticky=aOther.isSticky(i);
				if (isSticky)
				{	result.put(aOther.getKey(i), aOther.getValue(i),true);
				}
			}
			if (result.size()==0)
			{	result=RequestResponseFieldsWithStickyImpl.EMPTY;
			}
		}
		return result;
	}
	
	public RequestResponseFieldsWithStickyImpl(int aInitialSize)
	{	mData= new Object[aInitialSize*LEN];
	}
	
	public RequestResponseFieldsWithStickyImpl(IRequestResponseFields aOther)
	{	if (aOther instanceof RequestResponseFieldsWithStickyImpl)
		{	RequestResponseFieldsWithStickyImpl other = (RequestResponseFieldsWithStickyImpl)aOther;
			mIndex = other.mIndex;
			mData = new Object[other.mData.length];
			System.arraycopy(other.mData, 0, mData, 0, mIndex);
		}
		else
		{	int size=aOther.size();
			mIndex=size*LEN;
			mData = new Object[mIndex+LEN*2];
			int j=0;
			for (int i=0; i<size; i++)
			{	mData[j++]=aOther.getKey(i);
				mData[j++]=aOther.getValue(i);
				mData[j++]=aOther.isSticky(i);
			}
		}
	}
	
	/** Put in a new value, will replace all existing values with same key
	 * @param aIdentifier identifier to match the source request
	 * @param aRepresentation representation in response
	 */
	public void put(String aKey, Object aRepresentation)
	{	put(aKey,aRepresentation,Boolean.FALSE);
	}
	
	/** Put in a new value, will replace all existing values with same key
	 * @param aIdentifier identifier to match the source request
	 * @param aRepresentation representation in response
	 * @param aIsSticky should the header be sticky
	 */
	public void put(String aKey, Object aRepresentation, boolean aIsSticky)
	{	int putIndex=mIndex;
		for (int i=0; i<mIndex; i+=LEN)
		{	if (mData[i].equals(aKey))
			{	if (putIndex==mIndex)
				{	putIndex=i;
				}
				else
				{	//we have more than one
					remove(aKey);
					putIndex=mIndex;
					break;
				}
			}
		}
		if (putIndex==mIndex)
		{	//resize if needed
			if (mIndex==mData.length)
			{	int newSize=(mIndex==0)?LEN*2:mIndex*2;
				Object[] newData = new Object[newSize];
				System.arraycopy(mData, 0, newData, 0, mIndex);
				mData=newData;
			}
			mIndex+=3;
		}
		
		mData[putIndex++]=aKey;
		mData[putIndex++]=aRepresentation;
		mData[putIndex]=aIsSticky;
	}
	
	public void add(String aKey, Object aRepresentation)
	{	add(aKey,aRepresentation,Boolean.FALSE);
	}
		
	/** Add a new value, this will be added in addition to any existing values
	 * for the given key
	 * @param aKey
	 * @param aRepresentation
	 * @param aIsSticky should the header be sticky
	 */
	public void add(String aKey, Object aRepresentation, boolean aIsSticky)
	{
		//resize if needed
		if (mIndex==mData.length)
		{	int newSize=(mIndex==0)?LEN*2:mIndex*2;
			Object[] newData = new Object[newSize];
			System.arraycopy(mData, 0, newData, 0, mIndex);
			mData=newData;
		}
		mData[mIndex++]=aKey;
		mData[mIndex++]=aRepresentation;
		mData[mIndex++]=aIsSticky;
	}
	
	/** Remove first instance of the given key
	 * @param aKey
	 * @return the removed value
	 */
	public Object remove(String aKey)
	{	Object removed=null;
		for (int i=0; i<mIndex; i+=LEN)
		{	if (mData[i].equals(aKey))
			{	removed=mData[i+1];
				System.arraycopy(mData, i+LEN, mData, i,mIndex-i-LEN);
				mIndex-=LEN;
				break;
			}
		}
		return removed;
	}	

	public String getKey(int aIndex)
	{	return (String)mData[aIndex*LEN];
	}

	public Object getValue(int aIndex)
	{	return mData[aIndex*LEN+1];
	}

	public Object getValue(String aKey)
	{	Object result=null;
		for (int i=0; i<mIndex; i+=LEN)
		{	if (mData[i].equals(aKey))
			{	result=mData[i+1];
				break;
			}
		}	
		return result;
	}
	
	public List getValues(String aKey)
	{	List result = new ArrayList();
		for (int i=0; i<mIndex; i+=LEN)
		{	if (mData[i].equals(aKey))
			{	result.add(mData[i+1]);
			}
		}
		return result;
	}
	
	public int size()
	{	return mIndex/LEN;
	}
	
	public boolean equals(Object aOther)
	{	boolean equal = aOther instanceof RequestResponseFieldsWithStickyImpl;
		if (equal)
		{	RequestResponseFieldsWithStickyImpl other = (RequestResponseFieldsWithStickyImpl)aOther;
			equal=Arrays.equals(mData, other.mData);
		}
		return equal;
	}

	public boolean isSticky(int aIndex)
	{	return (Boolean)mData[aIndex*LEN+2];
	}

	public boolean isSticky(String aKey)
	{	boolean result=false;
		for (int i=0; i<mIndex; i+=LEN)
		{	if (mData[i].equals(aKey))
			{	result=(Boolean)mData[i+2];
				break;
			}
		}	
		return result;
	}

}
