/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.scheduler;

import java.util.*;

public class TraceDebugCapture
{
	private int mDepth=0;	
	private List<TraceEntry> mEntries;
	
	public TraceDebugCapture()
	{	mEntries=new ArrayList<TraceEntry>();
	}
	
	public void add(String aMessage)
	{	TraceEntry te=new TraceEntry(mDepth,0,null,aMessage);
		mEntries.add(te);
	}
	
	public void add(String aMessage, int aLevel, Object aEntity)
	{	TraceEntry te=new TraceEntry(mDepth,aLevel,aEntity,aMessage);
		mEntries.add(te);
	}
	
	public void insert(String aMessage, int aLevel, Object aEntity)
	{	
		/*
		TraceEntry te=new TraceEntry(mDepth,aLevel,aEntity,aMessage);
		mEntries.add(te);
		*/
		
		TraceEntry te=new TraceEntry(mDepth,aLevel,aEntity,aMessage);
		int i=mEntries.size()-1;
		while(i>0 && mEntries.get(i).getDepth()!=mDepth)
		{	i--;
		}
		if (i>=0)
		{	mEntries.set(i, te);
		}
			
	}

	public void push()
	{	mDepth++;
	}
	
	public void pop()
	{	mDepth--;
	}
		
	public List<TraceEntry> getEntries()
	{	return Collections.unmodifiableList(mEntries);
	}
	
	public void cloneFrom(TraceDebugCapture aOther)
	{	mEntries=aOther.mEntries;
		mDepth=aOther.mDepth;
	}
	
	public static class TraceEntry
	{
		private final int mDepth;
		private final int mLevel;
		private final Object mEntity;
		private final String mMessage;
		
		public TraceEntry(int aDepth, int aLevel, Object aEntity, String aMessage)
		{	mDepth=aDepth;
			mLevel=aLevel;
			mEntity=aEntity;
			mMessage=aMessage;
		}
		
		public int getDepth()
		{	return mDepth;
		}
		
		public int getLevel()
		{	return mLevel;
		}
		
		public Object getEntity()
		{	return mEntity;
		}
		
		public String getMessage()
		{	return mMessage;
		}
	}
}