/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.scheduler;

import org.netkernel.container.ILogger;

/** WorkerThread is a child of a WorkerThreadPool, it exists to provide
  execution threads to call its parents process() method
 */
public class WorkerThread extends Thread
{
    private WorkerThreadPool mParent;
    /** When true the thread is either signalled to stop ASAP or has already stopped.
     */
    private boolean mRequestStop;
	private boolean mIsStopped;
	private RequestState mLatest;
	
    /** Construct a new WorkerThread.
     * @param aParent The owning WorkerThreadGroup.
     */    
    public WorkerThread(WorkerThreadPool aParent, ThreadGroup aThreadGroup)
    {	super(aThreadGroup,aParent.getKernel().getLogger().format("WORKERTHREAD_NAME")+aParent.getThreadId());
		mParent = aParent;
    }
    
    /** The processing loop to do the threads work. It also checks the enabled and stop flags
     * to control the threads operations in a safe manner.
     */
    public void run()
    {  setContextClassLoader(getClass().getClassLoader());
	   while(!mRequestStop)
	   {	try	// keep catch block inside loop
			{	while (!mRequestStop)
				{	mParent.process();
					mLatest=null;
				}
			}
			catch (InterruptedException e)
			{   // natural termination if necessary
			}
			catch (Throwable e)
			{   // don't allow it to kill throws the thread just carry on
				mParent.getKernel().getLogger().log(ILogger.SEVERE, this, "WORKERTHREAD_EXCEPTION",e);
				e.printStackTrace();
			}
	   }
	   mIsStopped=true;
	   setContextClassLoader(null);
    }
    
    /** Signals for the thread to stop once it has finished any work it is part way
     * through.
     */
    public void requestStop()
    {   if (mRequestStop==false)
        {
    		mRequestStop=true;
            synchronized(this)
			{   notify();
			}
        }
    }	
	
	/** First attempts an interrupt, failing success the thread.stop() method is called which causes ThreadDeath
	 */
	@SuppressWarnings("deprecation")
	public void forceStop()
	{	this.interrupt();
		long timeout = mParent.getKernel().getConfiguration().getLong("netkernel.scheduler.thread.timeout",1000);
		long t0 = System.currentTimeMillis();
		while (System.currentTimeMillis()-t0<timeout && this.isAlive())
		{	try
			{	Thread.sleep(100);
			} catch (InterruptedException e) {;}
		}
		if (!mIsStopped)
		{	// force stop the thread if all attempts to interrupt it fail
			this.stop();
		}
	}
	
	public void setLatest(RequestState aLatest)
	{	mLatest=aLatest;
	}
	
	public RequestState getLatest()
	{	return mLatest;
	}
}