/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.urii.impl;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import org.netkernel.urii.INetKernelException;
import org.netkernel.urii.INetKernelThrowable;
import org.netkernel.util.Utils;

/**
 * NetKernel exception is a generic exception class used for most exceptions in the NetKernel. It is
 * nestable and allows multiple exceptions to be nested at any depth. Its most important characteristic
 * is that its state can be recursively dumped as XML giving a clear picture of an exception scenario is
 * a document.
 * @author  tab
 */
public class NetKernelException extends INetKernelException
{
	private static final long serialVersionUID = 7373551929231791843L;

	/** depth that java exception stack traces will be captured - configured by Config system component */
	private static int sTraceDepth=16;
	
	/** exception Id */
	private final String mId;
	
	/**
	 * Constructs an instance of <code>INetKernelException</code> with the specified detail message.
	 * @param aMessage the detail message.
	 */
	public NetKernelException(String aId)
	{	this(aId,null,null);
	}
	
	/**
	 * Constructs an instance of <code>INetKernelException</code> with the specified Id, detail message, and
	 * request id.
	 * @param aId an id for the exception
	 * @param aMessage the detail message.
	 */
	public NetKernelException(String aId, String aMessage)
	{	this(aId,aMessage,null);
	}
	
	/**
	 * Constructs an instance of <code>INetKernelException</code> with the specified Id, detail message, and
	 * request id.
	 * @param aId an id for the exception
	 * @param aMessage the detail message.
	 * @param aRequestId id of request that caused the exception
	 */
	public NetKernelException(String aId, String aMessage, Throwable aCause)
	{	super(aMessage,aCause);
		if (aId==null)
		{	mId=this.getClass().getName();
		}
		else
		{	mId = aId;
		}
	}
	
	/** @Return the id for the exception
	 */
	public String getId()
	{	return mId;
	}
	
	public boolean showNativeStack()
	{	return false;
	}

	public String getDeepestId()
	{	String result;
		Throwable deepest = getDeepestCause();
		if (deepest instanceof INetKernelThrowable)
		{	result = ((INetKernelThrowable)deepest).getId();
		}
		else
		{	result = deepest.getClass().getName();
		}	
		return result;
	}
	
	public String getDeepestMessage()
	{	Throwable deepest = getDeepestCause();
		return deepest.getMessage();
	}
	
	public Throwable getDeepestCause()
	{	Throwable result=this;
		while (result.getCause()!=null)
		{	result=result.getCause();
		}
		return result;
	}
	
	/** configure the depth that java exceptions stack traces will be captured
	 */
	public static void setTraceDepth(int aDepth)
	{	sTraceDepth = aDepth;
	}
	
	public static int getTraceDepth()
	{	return sTraceDepth;
	}
	
	/** Overriden to stop cause being set, this exception is immutable
	 */
	public Throwable initCause(Throwable aCause)
	{	throw new IllegalStateException();
	}
	
	public String toString()
	{	StringWriter sw = new StringWriter(1024);
		try
		{	Utils.writeThrowableAsXML(this, sw, 0);
			return sw.toString();
		}
		catch (IOException e)
		{	return super.toString();
		}
	}
	
	public void writeAdditionalFieldsAsXML(Writer aWriter, int aIndent) throws IOException
	{
	}

}