/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.util;

/** Fast add and contains methods with low memory overhead for a smallish dataset.
 *  This class is not threadsafe.
 * @author tab
 *
 */
public class ExclusionSet
{
	/** the set of objects added, duplicates are allowed */
	private Object[] mSet;
	/** the number of objects currently added */
	private int mSize;
	
	/** Construct a new ExclusionSet
	 * @param aInitialSize the initial capacity, the capacity will grow if needed
	 */
	public ExclusionSet(int aInitialSize)
	{	mSet=new Object[aInitialSize];
		mSize=0;
	}
	
	/** Check by identity to see if an object is already contained
	 * @param aObject the object to check
	 * @return true if the object is contained
	 */
	public boolean contains(Object aObject)
	{	int i;
		for (i=mSize-1; i>=0 && mSet[i]!=aObject; i--);
		return i>=0;
	}
	
	/** Add an item to the set, capacity will grow if necessary
	 * @param aObject the object to add
	 */
	public void add(Object aObject)
	{	if (mSize==mSet.length)
		{	Object[] newSet = new Object[mSet.length*2];
			System.arraycopy(mSet,0,newSet,0,mSet.length);
			mSet=newSet;
		}
		mSet[mSize++]=aObject;
	}
	
	public void clear()
	{	mSize=0;
	}
	
	public ExclusionSet takeCopy()
	{	ExclusionSet result = new ExclusionSet(mSet.length);
		result.mSize=mSize;
		System.arraycopy(mSet, 0, result.mSet, 0, mSize);
		return result;
	}
	
	public void restoreFromCopy(ExclusionSet aCopy)
	{	mSet=aCopy.mSet;
		mSize=aCopy.mSize;
	}
}